﻿using System;
using Ozeki.Network;
using Ozeki.VoIP;

namespace PBX_DialPlan
{
    class MainClass
    {
        private static void Main(string[] args)
        {
            var myPBX = new MyPBX(NetworkAddressHelper.GetLocalIP().ToString(), 20000, 30000);
            myPBX.Start();

            Console.ReadLine();
            myPBX.Stop();
        }
    }

    class MyDialPlanProvider : IDialplanProvider
    {
        public string Name { get; private set; }

        public IDialplanResult GetDestination(RouteInfo routeInfo)
        {
            // when extension "1002" dials any number, the call will be redirected to extension "1003"
            if (routeInfo.SourceExtension.ExtensionID == "1002")
                return new Destination("1003");

            // by default, call the number dialed by the caller
            return new Destination(routeInfo.DialInfo.Dialed); 
        }

        public SessionMode GetSessionMode(IBaseCall caller, IBaseCall callee, SessionMode defaultSessionMode)
        {
            return SessionMode.ForceP2P;
        }
    }

    class MyPBX : PBXBase
    {
        string localAddress;

        public MyPBX(string localAddress, int minPortRange, int maxPortRange)
            : base(minPortRange, maxPortRange)
        {
            this.localAddress = localAddress;
            Console.WriteLine("PBX starting...");
            Console.WriteLine("Local address: " + localAddress);
        }

        protected override void OnStart()
        {
            Console.WriteLine("PBX started.");
            SetListenPort(localAddress, 5060, TransportType.Udp);
            var callManager = GetService<ICallManager>();   // callManager object which handles dial plans
            callManager.DialplanProvider = new MyDialPlanProvider();    // the callManager will work with the modified dial plans
            
            Console.WriteLine("Listened port: 5060(UDP)");
            base.OnStart();
        }

        protected override RegisterResult OnRegisterReceived(ISIPExtension extension, SIPAddress from, int expires)
        {
            Console.WriteLine("Register received from: " + extension.ExtensionID);
            return base.OnRegisterReceived(extension, from, expires);
        }

        protected override void OnUnregisterReceived(ISIPExtension extension)
        {
            Console.WriteLine("Unregister received from: " + extension.ExtensionID);
            base.OnUnregisterReceived(extension);
        }

        protected override void OnCallRequestReceived(ISessionCall call)
        {
            Console.WriteLine("Call request received. Caller: " + call.DialInfo.CallerID + " callee: " + call.DialInfo.Dialed);
            base.OnCallRequestReceived(call);
        }
    }
}
